<?php
//** Get command-line argument **/ 
if (isset($argv[1])) {
    $arg = $argv[1];
    if ($arg === "--help") {
		help();
	}
}
global $apiuser, $apipasswd, $host, $days;
$apiuser = "admin";
$apipasswd = "opennac";
$host = "127.0.0.1";
$days = 30;
$options = getopt("u:p:h:d:");
getParameters($options);
$token = getAuthToken();
checkProfile($token);
logout($token);
//** Get the parameters based on the option values **//
function getParameters($options) {
	$showhelp = false;
	if (array_key_exists('u', $options)) {
		$GLOBALS['apiuser'] = $options["u"];
	}
	if (array_key_exists('p', $options)) {
		$GLOBALS['apipasswd'] = $options["p"];
	}
	if (array_key_exists('h', $options)) {
		if (filter_var($options["h"], FILTER_VALIDATE_IP) !== false) {
			$GLOBALS['host'] = $options["h"];
		} else {
			echo "\e[31mThe IP entered is not in the correct format\e[0m" . PHP_EOL;
			$showhelp = true;
		}
	}
	if (array_key_exists('d', $options)) {
		$filter_days = $options["d"];
		if (is_numeric($filter_days)) {
			$GLOBALS['days'] = (int)$filter_days;
		} else {
			echo "\e[31mThe days are not in the correct format\e[0m" . PHP_EOL;
			$showhelp = true;
		}
	}
	if ($showhelp) {
		help();
	}
}
//** Help menu of the script **//
function help()
{
	echo "\e[32m[Usage]\e[0m" . PHP_EOL;
	echo "\tScript to check the profile of user devices. For each UD, it checks if it complies at least one of the rules of the parent EPT. In case of incorrect compliance, writes UD information in a CSV file" . PHP_EOL;
	echo PHP_EOL . "\e[32m[Available options]\e[0m" . PHP_EOL;
	echo "\t-u\t Portal user (admin by default)" . PHP_EOL . "\t-p\t Portal password (opennac by default)" . PHP_EOL . "\t-h\t Host ip (127.0.0.1 by default)" . PHP_EOL .  "\t-d\t Limit of days to filter user devices by last modification date (30 by default)" . PHP_EOL; 
	echo PHP_EOL . "\e[32m[Execution example with all parameters]\e[0m" . PHP_EOL . "\tphp checkProfiling.php -u 'user1' -p '1234' -h '10.10.10.10' -d '60'" . PHP_EOL;
    echo PHP_EOL . "\e[32m[Example CSV file]\e[0m" . PHP_EOL;
	echo "\tMAC;EPT;Device tags;Matching rules;Parent EPT to modify\n";
	echo "\t00:14:38:48:98:C0;PRINTER_HP;DNS_HP_LASERJET,SHQ_HP_LASERJET,DOP_TCP_9000,MAC_001438;SSL,DNS;PRINTER\n";
	echo "\tA1:B2:C3:D4:E5:F6;CAMERA_LANACCESS;CNS_LANACCESS_WEB,COI_LANACCESS,DOP_UDP_53,MAC_A1B2C3;Certificate organization issuer,Certificate name subject;CAMERA\n";
	die();
}
//** Get the authentication token in case of correct credentials and url **//
function getAuthToken()
{
	$url = "http://" . $GLOBALS['host'] . "/api/auth";
	$fields = array("username" => $GLOBALS['apiuser'], "password" => $GLOBALS['apipasswd'], "useOnlyLocalRepo" => true);
	$curl = curl_init();
	curl_setopt_array(
		$curl,
		array(
			CURLOPT_URL => $url,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_CUSTOMREQUEST => "POST",
			CURLOPT_POSTFIELDS => json_encode($fields),
			CURLOPT_TIMEOUT => 10,
			CURLOPT_HTTPHEADER => array(
				"accept: application/json",
				"Content-Type: application/json"
			),
		)
	);
	$token = "";
	$response = curl_exec($curl);
	$result = json_decode($response, true);
	$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
	switch ($httpCode) {
		case 0:
			echo "\e[31mAuthentication token not obtained, reason: no route to " . $GLOBALS['host']. "\e[0m" . PHP_EOL;
			die();
		case 201:
			echo "\e[32mAuthentication token obtained\e[0m" . PHP_EOL;
			$token = @$result["token"];
			break;
		case 301:
			echo "\e[31mAuthentication token not obtained, reason: no route to " . $GLOBALS['host']. "\e[0m" . PHP_EOL;
			die();
		default:
			echo "\e[31mAuthentication token not obtained, reason: " . @$result["message"] . "\e[0m" . PHP_EOL;
			die();
	}
	curl_close($curl);
	return $token;
}
//** Check the complain of the parent profile rules **//
function checkParentProfile($mac, $ept, $deviceTags, $profilingRules, $parentRules, $parentProfile)
{
	$lines = [];
	$profiling = false;
	foreach ($parentRules as $rule) {
		if ($rule['result'] === true) {
			$profiling = true;
			break;
		}
	}
	// Get info of incorrect UD profiled
	if ($profiling === false) {
		$trueRules = [];
		foreach ($profilingRules as $ruleName => $rule) {
			if ($rule['result'] === true) {
				$trueRules[] = $ruleName;
			}
		}
		// Add incorrect UD profiled information
		$lines[] = array($mac, $ept, implode(',', $deviceTags), implode(',', $trueRules), $parentProfile);
	}
	return $lines;
}
//** Check the profiling for each UD **//
function checkProfile($token)
{
	$currentDate = date('Y-m-d');
	$daysToSubtract = $GLOBALS['days'];
	$filterDate = date('Y-m-d', strtotime("-$daysToSubtract days", strtotime($currentDate)));
    $incorrectProfiledUD = 0;
	$index = 0;
	$UD = 0; 
	$exit = false;
	$fileName = "checkProfiling_" . $filterDate. ".csv";
	// Delete old CSV file if is created
	if (file_exists($fileName)) {
		file_put_contents($fileName, '');
	}
	$headers = ['MAC','EPT','Device tags','Matching rules','Parent EPT to modify'];
	$hasWrittenHeaders = false;
	do {
		// Get CMDB User Devices
		$url = "http://" . $GLOBALS['host'] . "/api/deviceusermac?filters=modified%3E%3D" . $filterDate . "&start-index=" . $index . "&max-results=50";
		$curl = curl_init();
		curl_setopt_array(
				$curl,
				array(
					CURLOPT_URL => $url,
					CURLOPT_RETURNTRANSFER => true,
					CURLOPT_CUSTOMREQUEST => "GET",
					CURLOPT_HTTPHEADER => array(
						"accept: application/json",
						"X-Opennac-Token: " . $token,
						"X-Opennac-Username: " . $GLOBALS['apiuser']
					),
				)
		);
		$response = curl_exec($curl);
		$result = json_decode($response, true);
		$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
		curl_close($curl);
		switch ($httpCode) {
			case 200:
				if (empty($result)) {
					$exit = true;
					break;
				} 
				else {
                    $devices = [];
					// Get devices information to simulate
					foreach ($result as $ud) {
						$UD++;
						if ($ud["macaddress"] !== '00:00:00:00:00:00' && $ud["ept"] !== 'EPT_UNKNOWN' && !empty($ud["ept"])) {							
							$device = [
								"id" => $ud["id"],
								"name" => $ud["name"],
								"macaddress" => $ud["macaddress"],
								"tags" => $ud["tags"],
							];
                            $devices[] = $device;
                        }
                    }
					// Simulate User Devices profiling
					if($devices) {
						$url = "http://" . $GLOBALS['host'] . "/api/deviceuserprofilingsimulator";
						$curl = curl_init();
						$fields = array("action" => "profiling", "devices" => $devices);
						curl_setopt_array(
							$curl,
							array(
								CURLOPT_URL => $url,
								CURLOPT_RETURNTRANSFER => true,
								CURLOPT_CUSTOMREQUEST => "POST",
								CURLOPT_POSTFIELDS => json_encode($fields),
								CURLOPT_HTTPHEADER => array(
									"accept: application/json",
									"Content-Type: application/json",
									"X-Opennac-Token: " . $token,
									"X-Opennac-Username: " . $GLOBALS['apiuser']
								),
							)
						);
						$response = curl_exec($curl);
						$result = json_decode($response, true);
						$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
						curl_close($curl);
						if ($httpCode == 201) {
							if (!empty($result) && sizeof($result) == sizeof($devices)) {
								$csvLines = [];
								foreach ($result as $ud) {
									// Get UD current EPT profile
									$ept = '';
									$deviceTags = explode(',', $ud['deviceTags']);
									foreach ($deviceTags as $deviceTag) {
										if (strpos($deviceTag, 'EPT_') === 0) {
										  $ept =  substr($deviceTag, 4);;
										  break;
										}
									  }
									$prefixesToDelete = ['DPA_', 'PDP_', 'PP1_', 'PP2_', 'PP3_', 'PDP_'];
									foreach ($deviceTags as $key => $value) {
										foreach ($prefixesToDelete as $prefix) {
											if (strpos($value, $prefix) === 0) {
												unset($deviceTags[$key]);
												break;
											}
										}
									}
									$deviceTags = array_values($deviceTags);							
									$mac = $ud["deviceMac"]; 
									$insertTags = $ud["result"]["result"]["tags"]["insert"];
									foreach ($insertTags as $insertTag) {
										// Get parent profile (PP tag) which UD doesn´t complain any of its rules
										if (strpos($insertTag, 'PP') !== false) {
											$parentProfile = substr($insertTag, 4);
											// Discard if PP1 equals to EPT. Example: PP1_PRINTER, EPT_PRINTER
											if (strpos($insertTag, 'PP1_') !== false) {
												if (strcmp($parentProfile, $ept) === 0) {
													break;
												}
											}
											// Discard if PP2 equals to EPT. Example: PP2_PRINTER_HP, EPT_PRINTER_HP
											if (strpos($insertTag, 'PP2_') !== false) {
												if (strcmp($parentProfile, $ept) === 0) {
													break;
												}
											}
											// Discard if PP3 equals to EPT. Example: PP3_PRINTER_HP_LASERJET, EPT_PRINTER_HP_LASERJET
											if (strpos($insertTag, 'PP3_') !== false) {
												if (strcmp($parentProfile, $ept) === 0) {
													break;
												}
											}
											$profilingRules = $ud["result"]["trace"][$ept]["rules"];
											$parentProfilingRules = $ud["result"]["trace"][$parentProfile]["rules"];
											$info = checkParentProfile($mac, $ept, $deviceTags, $profilingRules, $parentProfilingRules, $parentProfile);
											if (!empty($info)) {
												$csvLines[] = $info;
												break;
											}
										}
									}
								}
								// Write incorrect UD profiled information in the CSV file
								if (!empty($csvLines)) {
									$fp = fopen($fileName, 'a');
									if (!$hasWrittenHeaders) {
										fwrite($fp, implode(';', $headers) . PHP_EOL);
										$hasWrittenHeaders = true;
									}
									foreach ($csvLines as $line) {
										fwrite($fp, implode(';', $line[0]) . PHP_EOL);
										$incorrectProfiledUD ++;
									}
									fclose($fp);
								}
							}
							else {
								echo "\e[31mSimulate profiling doesn´t return the same number of devices requested." ."\e[0m" . PHP_EOL;
							}
						}
						else {
							echo "\e[31mSimulate profiling not executed, reason: " . @$result["message"] . "\e[0m" . PHP_EOL;
						}
					}
                echo "\e[32m" . $UD . " user devices checked\e[0m" . PHP_EOL;
				}
        		break;	
			default:
				echo "\e[31mUnable to obtain ON CMDB User devices, reason: " . @$result["message"] . "\e[0m" . PHP_EOL;
				$exit = true;
				break;
		}
		$index = $index + 50;
	}  while ($exit == false);
	if ($incorrectProfiledUD > 0) {
		echo "\e[31m" . $incorrectProfiledUD . " incorrect user devices profiled\e[0m" . PHP_EOL;
	}
	else {
		echo "\e[32mThere aren´t incorrect user devices profiled whose last modification is later to " . $filterDate . "\e[0m" . PHP_EOL;
	}
}
//** Session logout **// 
function logout ($token) {
	$url = "http://" . $GLOBALS['host'] . "/api/auth/logout";
	$curl = curl_init();
	curl_setopt_array(
		$curl,
		array(
			CURLOPT_URL => $url,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_CUSTOMREQUEST => "DELETE",							
			CURLOPT_HTTPHEADER => array(
				"accept: application/json",
				"Content-Type: application/json",
				"X-Opennac-Token: " . $token,
				"X-Opennac-Username: " . $GLOBALS['apiuser']
			),
		)
	);
	$response = curl_exec($curl);
	$result = json_decode($response, true);
	$httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
	curl_close($curl);
	switch($httpCode) {
		case 204:
			echo "\e[32mSuccessfully logout". "\e[0m" . PHP_EOL;
			break;
		default:
		    echo "\e[31mError, can´t logout. Reason: ". @$result["message"] . "\e[0m" . PHP_EOL;
			break;
	}
}